# -*- coding: utf-8; tab-width: 4; mode: python -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: t -*
# vi: set ft=python sts=4 ts=4 sw=4 noet

#    pygifme is a simple command line tool to generate animated GIFs.
#    It is a python port from the original ruby script gifme created by
#    Zach Holman

#    Copyright (C) 2013 by Ángel Guzmán Maeso, shakaran at gmail dot com

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

__package__ = 'pygifme'
__version__ = '0.1'
__all__ = ['.pygifme']

def main(): # For setuptools entry point
    from . import pygifme
    pygifme.main()


#!/usr/bin/env python3
# -*- coding: utf-8; tab-width: 4; mode: python -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: t -*
# vi: set ft=python sts=4 ts=4 sw=4 noet

# pylint: disable-msg=R0903,C0103,C0301

#    pygifme is a simple command line tool to generate animated GIFs.
#    It is a python port from the original ruby script gifme created by
#    Zach Holman

#    Copyright (C) 2013 by Ángel Guzmán Maeso, shakaran at gmail dot com

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

# PYTHON_ARGCOMPLETE_OK

def main():
    from os import system, mkdir, listdir, environ, path, chdir, getcwd, \
                   EX_USAGE, EX_DATAERR, EX_CONFIG, EX_CANTCREAT
    from time import gmtime, strftime
    from sys import exit, path as spath
    from argparse import ArgumentParser, ONE_OR_MORE, HelpFormatter
    from argcomplete import autocomplete
    
    # Avoid problems with /usr/local/bin first in sys.path
    if path.dirname(__file__) == spath[0]:
        spath.append(spath[0])
        del spath[0]
    
    try:
        from pygifme import __version__
    except ImportError as e:
        #from . import __version__
        __version__ = '0.1' # @Fixme: Bad package import from /usr/local/bin/pygifme.py
        
    description = '  FILES can be listed out, like `file1.jpg file2.jpg`, or it\n' \
                  '  can be a normal shell glob, like `*.jpg`.'
            
    parser = ArgumentParser(
                             prog                  = 'pygifme',
                             description           = description,
                             epilog                = None,
                             parents               = [],
                             formatter_class       = HelpFormatter,
                             prefix_chars          = '-',
                             fromfile_prefix_chars = None,
                             argument_default      = None,
                             conflict_handler      = 'error',
                             add_help              = True,
                            )
    
    def valid_directory(directory):
        if path.exists(directory):
            if path.isdir(directory):
                return directory
            else:
                parser.error('Path {0} is not a directory'.format(directory))
                return directory
        else:
            parser.error('Directory path {0} does not exist'.format(directory))
                       
    parser.add_argument('-r', '--reverse',
                        action         = 'store_true',
                        dest           = 'reverse',
                        default        = False,
                        help           = 'Reverse the GIF to make it loopable')
    
    parser.add_argument('-o', '--output',
                        action         = 'store',
                        metavar        = '/path/to/output',
                        type           = lambda d:valid_directory(d),
                        dest           = 'output',
                        choices       = None,
                        help           = 'Set the animation\'s output directory')
    
    parser.add_argument('-d', '--delay',
                        action         = 'store',
                        metavar        = 'DELAY',
                        dest           = 'delay',
                        default        = 20,
                        type           = int,
                        choices        = None,
                        help           = 'Set the delay between frames (default: 20)')
    
    parser.add_argument('-w', '--width',
                        action         = 'store',
                        metavar        = 'PIXELS',
                        dest           = 'width',
                        default        = 500,
                        type           = int,
                        choices        = None,
                        help           = 'Set the width of the image (default: 500px)')
    
    parser.add_argument('-q', '--quiet',
                        action         = 'store_true',
                        dest           = 'quiet', 
                        default        = False,
                        help           = 'Don\'t print status messages to stdout')
    
    parser.add_argument(option_strings = ['FILES'], 
                        metavar        = 'FILES', 
                        nargs          = ONE_OR_MORE,
                        type=str,
                        dest           = 'FILES',
                        help           = 'One or more files to process')
    
    parser.add_argument('-v', '--version', 
                        action  ='version',
                        version ='%(prog)s {version}'.format(version = __version__),
                        help    = 'Shows the program version')
    
    autocomplete(parser)
    args = parser.parse_args()
    
    #print(vars(args)) # For debugging
    
    if system("which convert 2>&1 > /dev/null") != 0:
        parser.error('You need to install ImageMagick first.\n\n' \
               'If you\'re on GNU/LINUX Debian systems, this should be as easy as:\n'\
               '  sudo apt-get install imagemagick\n' \
               'If you\'re on a Mac, this should be as easy as:\n' \
               '  brew install imagemagick')
        exit(EX_CONFIG)
        
    if not args.FILES: # no files given
        parser.error('no FILES given to process')
        exit(EX_USAGE)
    else:
        for pfile in args.FILES:
            if pfile[0:4] != 'http': # skip remote files
                if path.exists(pfile):
                    if not path.isfile(pfile):
                        parser.error('{0} is not a valid file'.format(pfile))
                else:
                    import glob
                    result_glob = glob.glob(pfile)
                    if not result_glob:
                        parser.error('File {0} does not exist'.format(pfile))
                    else:
                        for gfile in result_glob:
                            if path.exists(gfile):
                                if not path.isfile(gfile):
                                    parser.error('{0} is not a valid file'.format(gfile))
                            else:
                                parser.error('File {0} does not exist'.format(gfile))
        
    # WORKING WITH REMOTE FILES
    if args.FILES[0][0:4] == 'http':
        from urllib import request
        from shutil import rmtree
        
        if path.exists('/tmp/pygifme'):
            rmtree('/tmp/pygifme')
            
        mkdir('/tmp/pygifme', 0o777)
        
        local_path = '/tmp/downloaded-pygifme.gif'
    
        with open(local_path, mode='wb') as wfile:
            remote_file = request.urlopen(args.FILES[0]).read()
            wfile.write(remote_file)
    
        wd = getcwd()
        chdir('/tmp/pygifme')
        status = system('convert {0} -coalesce frame_%03d.gif'.format(local_path))
        chdir(wd)
        
        if status != 0:
            parser.error('Could not process remote file {0}'.format(local_path))
            exit(EX_DATAERR)
        
        del args.FILES[0]
        args.FILES += [path.join('/tmp/pygifme/', f) for f in listdir('/tmp/pygifme/')]
    
    if args.reverse:
        args.FILES += args.FILES[1:-2]
    
    if not args.output:
        import subprocess
        desktop = str(subprocess.check_output(['xdg-user-dir', 'DESKTOP'], universal_newlines = True)).strip()
        
        if not desktop:
            home = environ.get('HOME')
            if not home:
                home = '/tmp/pygifme'
                mkdir(home, 0o777)
            else:
                home = '{0}/Desktop'.format(home)
        else:
            home = desktop

        args.output = home
    
    args.output = str(args.output) + '/animated-{0}.gif'.format(strftime('%F_%Hh-%Mm-%Ss', gmtime()))
               
    cmd = 'convert -delay {0} -loop 0 -resize {1} -layers OptimizeTransparency {2} {3}'.format(int(args.delay), int(args.width), ' '.join(args.FILES), str(args.output))
    if system(cmd) == 0:
        if not args.quiet:
            print('You now have a handsome animation at {0}'.format(args.output))
    else:
        parser.error('Something broke when we were animating your gif. Shit.')
        exit(EX_CANTCREAT)
    
    if system('which cloudapp 2>&1 > /dev/null') == 0 and not environ.get('DISABLE_CLOUPAPP'):
        if not args.quiet:
            print('Now we\'re uploading it to CloudApp')
        system('cloudapp {0}'.format(args.output))

if __name__ == '__main__':
    main()


#!/usr/bin/env python3
# -*- coding: utf-8; tab-width: 4; mode: python -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: t -*
# vi: set ft=python sts=4 ts=4 sw=4 noet 

#    pygifme is a simple command line tool to generate animated GIFs
#    It is a python port from the original ruby script gifme created by
#    Zach Holman

#    Copyright (C) 2013 by Ángel Guzmán Maeso, shakaran at gmail dot com

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

import sys

if sys.version_info[:3] < (3, 0, 0):
    print('Python 3.0 or higher is required, {0} found. Aborting.'.format('.'.join(map(str, sys.version_info[:3]))))
    sys.exit(1)

print("Python starts at " + sys.prefix)

import platform
print ('Distribution:', ' '.join(map(str, platform.dist())))

__long_description__  = """pygifme is a simple command line tool to generate animated GIFs.
It is a python port from the original ruby script gifme created by Zach Holman"""

__classifiers__       = [
                            'Development Status :: 3 - Alpha',
                            'Environment :: Console',
                            'Environment :: MacOS X',
                            'Environment :: Web Environment',
                            'Intended Audience :: End Users/Desktop',
                            'Intended Audience :: Developers',
                            'Intended Audience :: System Administrators',
                            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
                            'Natural Language :: English',
                            'Operating System :: POSIX',
                            'Operating System :: POSIX :: Linux',
                            #'Operating System :: Microsoft :: Windows', # Not yet
                            'Operating System :: MacOS',
                            'Operating System :: MacOS :: MacOS X',
                            'Programming Language :: Python',
                            'Programming Language :: Python :: 3',
                            'Programming Language :: Python :: 3.0',
                            'Programming Language :: Python :: 3.1',
                            'Programming Language :: Python :: 3.2',
                            'Programming Language :: Python :: 3.3',
                            'Programming Language :: Python :: Implementation',
                            'Topic :: Multimedia :: Graphics :: Graphics Conversion',
                            'Topic :: Utilities',
                       ]

__data_files__ = [
                  ('/usr/share/doc', ['README.md']),
                 ]

try:
    from distutils.core import setup
    from distutils.core import Command
    
    from unittest import TestLoader, TextTestRunner
    from doctest import DocTestSuite
    
    class Test(Command):
        description = 'run unit tests and doc tests'
    
        user_options = []
    
        def initialize_options(self):
            pass
    
        def finalize_options(self):
            pass
    
        def run(self):
            pynames = ['test']
    
            # Add unit-tests:
            loader = TestLoader()
            suite = loader.loadTestsFromNames(pynames)
    
            # Add doc-tests:
            for name in pynames:
                suite.addTest(DocTestSuite(name))
    
            # Run the tests:
            runner = TextTestRunner(verbosity=2)
            result = runner.run(suite)
            if not result.wasSuccessful():
                raise SystemExit(2)
    
    from os.path import join as pjoin
    
    kw = {
          'scripts': [pjoin('bin', 'pygifme'), 'pygifme.py'], 
          'cmdclass' : {
                        'test' : Test, # Test support only for distutils (no setuptools)
                       }
         }
except ImportError:
    from setuptools import setup
    kw = {'entry_points':
          """[console_scripts]\npygifme = pygifme:main\n""",
          'zip_safe': False}

setup(
      name             = 'pygifme',
      version          = '0.1',
      description      = 'pygifme is a simple command line tool to generate animated GIFs.',
      long_description = __long_description__,
      author           = 'Ángel Guzmán Maeso',
      author_email     = 'shakaran@gmail.com',
      maintainer       = 'Ángel Guzmán Maeso',
      maintainer_email = 'shakaran@gmail.com',
      url              = 'https://github.com/shakaran/pygifme',
      download_url     = 'https://github.com/shakaran/pygifme',
      packages         = ['pygifme'],
      package_dir      = {'pygifme': '.'},
      package_data     = {'pygifme': ['setup.py', 'test.py']},
      license          = 'GNU GPL Version 3',
      platforms        = 'Python 3.0 and later',
      classifiers      = __classifiers__,
      keywords         = [],
      data_files       = __data_files__,
      py_modules       = ['pygifme'],
      requires         = ['unitest', 'doctest', 'imagemagick', 'argcomplete'], #cloudapp (ruby) (optional)
      provides         = [],
      obsoletes        = [],
      **kw
)


# -*- coding: utf-8; tab-width: 4; mode: python -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: t -*-
# vi: set ft=python sts=4 ts=4 sw=4 noet 

#    pygifme is a simple command line tool to generate animated GIFs. 
#    It is a python port from the original ruby script gifme created by
#    Zach Holman

#    Copyright (C) 2013 by Ángel Guzmán Maeso, shakaran at gmail dot com

#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
import sys
from os import system, environ, EX_OK
from subprocess import check_output, CalledProcessError, STDOUT
import unittest

if sys.version_info[:3] < (3, 0, 0):
    print('Python 3.0 or higher is required, {0} found. Aborting.'.format('.'.join(map(str, sys.version_info[:3]))))
    sys.exit(1)

def disable_cloud_app():
    environ['DISABLE_CLOUPAPP'] = '1'
    
def enable_cloud_app():
    if environ.get('DISABLE_CLOUPAPP'):
        del environ['DISABLE_CLOUPAPP']

class TestBinaries(unittest.TestCase):
    """ Test Binaries """
        
    @unittest.skipIf(sys.version_info < (2, 6),
                     'argparse not supported in this python version')
    def test_argpase(self):
        """ Checks the presence of argparse module """
        try:
            import argparse
            assert True
        except ImportError as e:
            self.fail('The module argparse is required')
       
    def test_argcomplete(self):
        """ Checks the presence of argcomplete module """
        try:
            import argcomplete
            assert True
        except ImportError as e:
            self.fail('The module argcomplete is required. Try: sudo pip install argcomplete')
                 
    def test_binary(self):
        """ Checks the binary presence of pygifme """
        assert system('which pygifme 2>&1 > /dev/null') == 0
        
    def test_imagemagick(self):
        """ Checks the binary presence of imagemagick """
        assert system('which convert 2>&1 > /dev/null') == 0
        
    def test_cloudapp(self):
        """ Checks the binary presence of cloudapp """
        # Detect presence of cloudapp (no required at all)
        # gem install cloudapp
        if system('which cloudapp 2>&1 > /dev/null') == 0:
            self.assertTrue(True, 'cloudapp binary detected')
        else:
            self.skipTest('cloudapp no detected. Skipping because not mandatory')

class TestArguments(unittest.TestCase):
    """ Test Arguments """
    
    REMOTE_GIF = 'https://a248.e.akamai.net/camo.github.com/0b546aee911bb34aeaa3109081ec70d15b159d2c/687474703a2f2f662e636c2e6c792f6974656d732f3054306632773243327a335433343377307533372f7075672e676966'
    
    def setUp(self):
        disable_cloud_app()
    
    def tearDown(self):
        enable_cloud_app()
             
    def test_no_files(self):
        """ Test invocation without FILES argument """
        
        try:
            check_output(['pygifme'], stderr = STDOUT)
            self.fail('error: pygifme must run with FILES argument')
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            result = 'the following arguments are required: FILES'
            self.assertIn(result, str(e.output))
    
    def test_unexistant_file(self):
        """ Test invocation with unexistant FILES argument """
        try:
            output = check_output(['pygifme', 'some_unexistant_file'], stderr = STDOUT)
            self.fail('Failed because the file should be unexistant:' + str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            
            result = 'does not exist'
            self.assertIn(result, str(e.output))
            
        
    def test_one_file(self):
        """ Test invocation with one FILES argument """
        output = check_output(['pygifme', 'test/1.jpg'], stderr = STDOUT)
        
        result = 'You now have a handsome animation at'
        self.assertIn(result, str(output))
    
    def test_two_files(self):
        """ Test invocation with two FILES argument """
        output = check_output(['pygifme', 'test/1.jpg', 'test/2.jpg'], stderr = STDOUT)
                              
        result = 'You now have a handsome animation at'
        self.assertIn(result, str(output))

    def test_global_files(self):
        """ Test invocation with global FILES argument """
        try:
            output = check_output(['pygifme', 'test/*.jpg'], stderr = STDOUT)
            
            result = 'You now have a handsome animation at'
            self.assertIn(result, str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            self.fail('Failed because:' + str(e.output))
    
    def test_remote_file(self):
        """ Test invocation with remote FILES argument """
        try:
            output = check_output(['pygifme', self.REMOTE_GIF], stderr = STDOUT)
                                  
            result = 'You now have a handsome animation at'
            self.assertIn(result, str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            self.fail('Failed because:' + str(e.output))
    
    def test_missing_delay(self):
        """ Test invocation with missing DELAY argument """
        try:
            output = check_output(['pygifme', '-d'], stderr = STDOUT)
            self.fail('Failed because the file should be unexistant:' + str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            
            result = 'expected one argument'
            self.assertIn(result, str(e.output))
        
        try:
            output = check_output(['pygifme', '-d', 'test/1.jpg'], stderr = STDOUT)
            self.fail('Failed because the file should be unexistant:' + str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            
            result = 'invalid int value'
            self.assertIn(result, str(e.output))
    
    def test_delay(self):
        """ Test invocation with DELAY argument """
        try:
            output = check_output(['pygifme', '-d', '25', 'test/1.jpg'], stderr = STDOUT)
            
            result = 'You now have a handsome animation at'
            self.assertIn(result, str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            self.fail('Failed because:' + str(e.output))
    
    def test_missing_width(self):
        """ Test invocation with missing WIDTH argument """
        try:
            output = check_output(['pygifme', '-w'], stderr = STDOUT)
            self.fail('Failed because the file should be unexistant:' + str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            
            result = 'expected one argument'
            self.assertIn(result, str(e.output))
        
        try:
            output = check_output(['pygifme', '-w', 'test/1.jpg'], stderr = STDOUT)
            self.fail('Failed because the file should be unexistant:' + str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            
            result = 'invalid int value'
            self.assertIn(result, str(e.output))
    
    def test_with(self):
        """ Test invocation with WITH argument """
        try:
            output = check_output(['pygifme', '-w', '200', 'test/1.jpg'], stderr = STDOUT)
            
            result = 'You now have a handsome animation at'
            self.assertIn(result, str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            self.fail('Failed because:' + str(e.output))
            
    def test_reverse(self):
        """ Test invocation with -r/--reverse argument """
        try:
            output = check_output(['pygifme', '-r', 'test/1.jpg', 'test/2.jpg'], stderr = STDOUT)
            
            result = 'You now have a handsome animation at'
            self.assertIn(result, str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            self.fail('Failed because:' + str(e.output))
    
    def test_ouput(self):
        """ Test invocation with -o/--output argument """
        try:
            output = check_output(['pygifme', '-o', '/tmp/pygifme', 'test/1.jpg', 'test/2.jpg'], stderr = STDOUT)
            
            result = 'You now have a handsome animation at'
            self.assertIn(result, str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            self.fail('Failed because:' + str(e.output))
    
    def test_quiet(self):
        """ Test invocation with -q/--quiet argument """
        try:
            output = check_output(['pygifme', '-q', 'test/1.jpg', 'test/2.jpg'], stderr = STDOUT)
            
            result = ''
            self.assertIn(result, str(output))
        except CalledProcessError as e:
            self.assertNotEqual(e.returncode, EX_OK)
            self.fail('Failed because:' + str(e.output))
    
    def test_help(self):
        """ Test invocation with -h/--help argument """
        output = check_output(['pygifme', '-h'], stderr = STDOUT)
        
        result = 'usage: pygifme'
        self.assertIn(result, str(output))
        
        output = check_output(['pygifme', '--help'], stderr = STDOUT)
        
        self.assertIn(result, str(output))
